@@ -27,6 +27,8 @@
 ;;; Code:
 
 (require 'blue)
+(require 'blue-complete)
+
 (require 'cl-generic)
 (require 'cus-edit) ; For `custom-button' face.
 (require 'eieio)
@@ -116,6 +118,9 @@ can become key characters."
 
 This is used to create the `blue-transient' menu.")
 
+;; FIXME: if this holds the menu, it should be resetted everytime this module is
+;; reloaded, that will avoid the inconsistent state where the transient does not
+;; get evaluated and remains with the stub definition.
 (defvar blue-transient--menu-expresion nil
   "Last evaluated transient.
 
@@ -622,31 +627,29 @@ to be specially handled."
    (lambda (category)
      (let* ((category-name (car category))
             (category-commands (cdr category))
-            (category-command-names (mapcar (lambda (command)
-                                              (alist-get 'invoke command))
-                                            category-commands))
-            (category-command-keys (blue-transient--assign-keys category-command-names nil)))
+            (category-command-invocations (blue--get-command-invocations category-commands))
+            (category-command-keys (blue-transient--assign-keys category-command-invocations nil)))
        (append
         (list (capitalize category-name))
         (mapcar
          (lambda (command)
-           (let* ((command-invoke (alist-get 'invoke command))
+           (let* ((command-name (symbol-name (car command)))
                   (command-key
                    (if (> (length category-commands) 1)
                        (concat (cadr (assoc category-name category-keys))
-                               (cadr (assoc command-invoke category-command-keys)))
-                     (cadr (assoc command-invoke category-command-keys))))
-                  (command-synopsis (alist-get 'synopsis command)))
+                               (cadr (assoc command-name category-command-keys)))
+                     (cadr (assoc command-name category-command-keys))))
+                  (command-synopsis (blue--command-get-slot 'synopsis command)))
              `(,command-key
-               ,(capitalize command-invoke)
+               ,(capitalize command-name)
                (lambda () ,command-synopsis (interactive)
                  (blue-transient--save-state)
                  (let ((command-chain (if blue-transient--command-chain
                                           (blue-transient--insert-nth
                                            (1+ blue-transient--selected-index)
-                                           '(,command-invoke)
+                                           '(,command-name)
                                            blue-transient--command-chain)
-                                        '((,command-invoke)))))
+                                        '((,command-name)))))
                    (setq blue-transient--command-chain command-chain)
                    (blue-transient--command-index+1 t)
                    (blue-transient--set-and-setup))))))
@@ -655,20 +658,20 @@ to be specially handled."
 
 (defun blue-transient--build-menu (commands)
   "Build transient menu for BLUE COMMANDS."
-  (let* ((category-names (seq-uniq (mapcar (lambda (command)
-                                             (alist-get 'category command))
-                                           commands)))
-         (category-keys (blue-transient--assign-keys category-names t))
-         (sorted-commands-by-category
-          (mapcar (lambda (category)
-                    (cons category
-                          (seq-filter (lambda (command)
-                                        (string-equal (alist-get 'category command)
-                                                      category))
-                                      commands)))
-                  category-names))
+  (let* ((categories (blue--get-command-categories commands))
+         (category-keys (blue-transient--assign-keys categories t))
+         (commands-by-category
+          (mapcar
+           (lambda (category)
+             (cons category
+                   (seq-filter
+                    (lambda (command)
+                      (string-equal (blue--command-get-slot 'category command)
+                                    category))
+                    commands)))
+           categories))
          (grouped-commands
-          (blue-transient--group-commands sorted-commands-by-category
+          (blue-transient--group-commands commands-by-category
                                           category-keys)))
     ;; Make each menu entry a vector.
     (mapcar (lambda (item)
@@ -713,29 +716,55 @@ suffixes."
             (min blue-transient--selected-argument-index
                  (1- (length selected-command-args)))))))
 
-(defun blue-transient--arguments-menu (command)
-  "Build transient menu for BLUE COMMAND arguments."
-  (when-let* ((commands-data (car blue--data))
-              (command-data (seq-find (lambda (cmd)
-                                        (equal (alist-get 'invoke cmd) command))
-                                      commands-data))
-              (completion-table (alist-get 'completion-table command-data))
-              (suffixes (seq-filter (lambda (suffix)
-                                      (string-prefix-p "--" suffix))
-                                    completion-table))
-              (suffixes-keys (blue-transient--assign-keys suffixes nil))
-              (menu-entries (mapcar (lambda (pair)
-                                      (let* ((suffix (car pair))
-                                             (key (cadr pair))
-                                             (msg (string-replace
-                                                   "-" " " (string-trim-left suffix "--")))
-                                             (msg* (capitalize msg)))
-                                        `(,(concat "--" key)
-                                          ,msg*
-                                          ,(concat suffix "=")
-                                          :class blue-transient--command-argument)))
-                                    suffixes-keys)))
-    menu-entries))
+;; TODO: Use the autocomplete option slot to provide the apropriate argument
+;; interface.
+(defun blue-transient--argument-menu-entry (key option)
+  "Create a transient argument entry from BLUE option."
+  (when-let* ((long-label (car (blue--get-option-long-labels option))))
+    (let* ((doc (alist-get 'doc option))
+           (arguments (alist-get 'arguments option))
+           (type (alist-get 'type arguments))
+           (trans-label (concat "--"
+                                long-label
+                                (if (string= type "required")
+                                    "="
+                                  " ")))
+           (autocomplete (alist-get 'autocomplete option))
+           (reader (cond
+                    ((string-equal autocomplete "directory")
+                     #'transient-read-directory)
+                    ((string-equal autocomplete "file")
+                     #'transient-read-file)
+                    ;; TODO:
+                    ;; ((and (string-equal autocomplete "set")
+                    ;;       table)
+                    ;;  `( ,(point) ,(point)
+                    ;;     ,table
+                    ;;     :exclusive 'no
+                    ;;     :company-kind (lambda (_) 'property)
+                    ;;     :company-doc-buffer ,doc-buffer-function
+                    ;;     :affixation-function ,affixation-function))
+                    )))
+      `(,(concat "--" key)
+        ,(capitalize long-label)
+        ,trans-label
+        :summary ,doc
+        :reader ,reader
+        :class blue-transient--command-argument))))
+
+(defun blue-transient--arguments-menu (command-name)
+  "Build transient menu for BLUE COMMAND-NAME arguments."
+  (when-let* ((commands (car blue--data))
+              (command (blue--get-command (intern command-name) commands))
+              (options (blue--command-get-slot 'options command))
+              (option-labels (seq-mapcat #'blue--get-option-long-labels options))
+              (option-keys (blue-transient--assign-keys option-labels nil))
+              (entries (mapcar (lambda (option)
+                                 (when-let* ((label (car (blue--get-option-long-labels option)))
+                                             (key (cadr (assoc-string label option-keys))))
+                                   (blue-transient--argument-menu-entry key option)))
+                               options)))
+    entries))
 
 
 ;;; Selected build directory.
@@ -763,7 +792,7 @@ This function is meant for side effects, it is responsible of keeping
   ;; NOTE: `corfu--minibuffer-on` won't enable `corfu-mode'
   ;; if `completion-at-point-functions` isn't local.
   (add-hook 'completion-at-point-functions
-            #'blue--completion-at-point nil t))
+            #'blue-completion-at-point nil t))
 
 (defun blue-transient--prompt-args ()
   "Helper for prompting BLUE command arguments."
@@ -818,10 +847,10 @@ This function is meant for side effects, it is responsible of keeping
      'transient--prefix
      '([(:info (propertize "No arguments" 'face 'shadow) :format "%d")]))))
 
-(defun blue-transient--menu ()
-  "Silence bytecompilation warnings.
-
-This will get redefined by `blue-transient'.")
+;; This function is defined at run-time during the evaluation of
+;; `blue-transient'. Let's declare it here to silence the byte-compilation
+;; warning.
+(declare-function blue-transient--menu "blue-transient" ())
 
 ;;;###autoload
 (defun blue-transient ()
@@ -848,7 +877,8 @@ keeps running in the compilation buffer."
   (blue--ensure-cache)
   (setq blue-transient--history-index 0
         blue--blueprint (blue--find-blueprint))
-  (let* ((prefix (car current-prefix-arg))
+  (let* ((blue-transient-menu 'blue-transient--menu)
+         (prefix (car current-prefix-arg))
          (build-dirs (blue--cache-get-build-dirs blue--blueprint))
          (last-build-dir (car build-dirs))
          (prompt-dir-p (or (eql prefix 4) ; Single universal argument 'C-u'.
@@ -870,7 +900,7 @@ keeps running in the compilation buffer."
            ;; previous lines.
            (build-dirs (blue--cache-get-build-dirs blue--blueprint))
            (indices (number-sequence 1 (length build-dirs)))
-           (transient `(transient-define-prefix blue-transient--menu ()
+           (transient `(transient-define-prefix ,blue-transient-menu ()
                          :incompatible '(,(cons "--build-directory="
                                                 build-dirs))
                          :init-value
@@ -921,7 +951,7 @@ keeps running in the compilation buffer."
                            ("-a" "Always build" "--always-build")
                            (5 "-c" "Color" "--color="
                               :choices ("auto" "always" "never"))
-                           (5 "-C" "Compiled load path" "--compiled-load-path"
+                           (5 "-C" "Compiled load path" "--compiled-load-path="
                               :prompt "Compiled load path: "
                               :reader transient-read-existing-directory)
                            (5 "-F" "File" "--file="
@@ -1035,7 +1065,7 @@ keeps running in the compilation buffer."
         (setq blue-transient--menu-expresion transient)
         (eval transient))
       ;; Open menu.
-      (blue-transient--menu))))
+      (funcall blue-transient-menu))))
 
 (provide 'blue-transient)
 ;;; blue-transient.el ends here.
