@@ -3,8 +3,8 @@
 ;; Copyright © 2025 Sergio Pastor Pérez
 ;;
 ;; Author: Sergio Pastor Pérez <sergio.pastorperez@gmail.com>
-;; Package-Version: 20251119.2110
-;; Package-Revision: 9aaf48ee5cb9
+;; Package-Version: 20251223.1409
+;; Package-Revision: b2778111dba4
 ;; Package-Requires: ((emacs "30.1") (magit-section "4.3.8"))
 ;; Keywords: blue, tools
 ;; URL: https://codeberg.org/lapislazuli/blue.el
@@ -355,7 +355,7 @@ If NO-SAVE is non-nil, don't save to disk immediately."
 If RAW is non nil, the serialized string will not be evaluated."
   (let* ((default-directory (or (blue--get-build-dir) default-directory))
          (process-environment (cons "GUILE_AUTO_COMPILE=0" process-environment))
-         (args (append (or options '()) (append '("--color" "always") commands)))
+         (args (append (or options '()) (cons "--color=always" commands)))
          (command-string (string-join (cons blue-binary args) " "))
          exit-code
          (raw-output (with-output-to-string
@@ -369,7 +369,7 @@ If RAW is non nil, the serialized string will not be evaluated."
 (defun blue--get-data (blueprint)
   "Return the commands and execution environment provided by BLUEPRINT."
   (when-let* ((options (when blueprint
-                         (list "--file" blueprint)))
+                         (list (concat "--file=" blueprint))))
               (cmd '(".serialize-commands" "--" ".serialize-execution-environment"))
               (output (blue--execute-deserialize options cmd))
               (data (car output))
@@ -379,7 +379,7 @@ If RAW is non nil, the serialized string will not be evaluated."
           (insert data)
           (goto-char (point-min))
           (condition-case err
-              (re-search-forward "\\(\\[.*\\]\\)\\({.*}\\)")
+              (re-search-forward "\\({.*}\\)\\({.*}\\)")
             (search-failed
              (blue--log-output (error-message-string err) "[BLUE] `re-search-forward'" 1)
              (with-current-buffer blue--log-buffer
@@ -407,128 +407,40 @@ If RAW is non nil, the serialized string will not be evaluated."
   "Retrieve variable VAR value from CONFIG."
   (cdr (assoc-string var config)))
 
-
-;;; Completion.
+(defun blue--get-command (command commands)
+  "Retrieve COMMAND from COMMANDS."
+  (assoc command commands))
 
-(blue--define-memoized blue--autocomplete (blueprint input)
-  "Use blue '.autocomplete' command to provide completion from INPUT."
-  (let* ((default-directory (or (blue--get-build-dir) default-directory))
-         (command (concat blue-binary
-                          " --file " blueprint
-                          " .autocomplete \"blue " input "\""))
-         (output (shell-command-to-string command)))
-    (string-split output)))
+(defun blue--command-get-slot (slot command)
+  "Retrieve SLOT from COMMAND."
+  (let ((fields (alist-get 'fields command)))
+    (alist-get slot fields)))
 
-(defun blue--completion-table (&rest _)
-  "Completion table function for minibuffer prompt."
-  (let ((result
-         (while-no-input
-           (when-let* ((blueprint (or blue--blueprint
-                                      (blue--find-blueprint)))
-                       (prompt-start (minibuffer-prompt-end))
-                       (input (buffer-substring prompt-start (point)))
-                       (completions (blue--autocomplete blueprint input)))
-             completions))))
-    (and (consp result) result)))
+(defun blue--get-command-invocations (commands)
+  "Retrieve command names from COMMANDS."
+  (mapcar (lambda (command)
+            (symbol-name (car command)))
+          commands))
 
-(defun pcomplete/blue ()
-  "Completion for `blue'."
-  (while (pcomplete-here* (blue--completion-table))))
+(defun blue--get-command-categories (commands)
+  "Retrieve command categories from COMMANDS."
+  (seq-uniq (mapcar (lambda (command)
+                      (blue--command-get-slot 'category command))
+                    commands)))
 
-(defun blue--completion-at-point ()
-  "`completion-at-point' function for `blue-run-command'."
-  (pcase (bounds-of-thing-at-point 'symbol)
-    (`(,beg . ,end)
-     ;; `beg-no-prompt' is required to ensure that completion receives the
-     ;; correct user input bounds even for prompts that do not leave any
-     ;; whitespace between the prompt and the user input. For example, the
-     ;; `transient-infix' default prompts, eg.: 'output=...'.
-     (let ((beg-no-prompt (max beg (minibuffer-prompt-end))))
-       (list beg-no-prompt end
-             (completion-table-with-cache #'blue--completion-table)
-             :exclusive 'no)))))
+(defun blue--get-option-long-labels (option)
+  "Retrieve long lable from OPTION."
+  (when-let* ((labels (alist-get 'labels option))
+              (long-labels (alist-get 'long labels)))
+    long-labels))
 
-
-;;; Minibuffer Hints.
-
-(defun blue--format-build-dir-hint (index build-dir current)
-  "Format a single build directory hint line.
-
-INDEX is the number prefixing the displayed known build directory in the
-hint message.
-BUILD-DIR is the directory of the known build directory.
-If CURRENT is non-nil the entry will be highlighted."
-  (let ((face (if (string-equal build-dir current)
-                  'blue-hint-highlight
-                'blue-hint-faded))
-        (build-dir* (buttonize build-dir
-                               (lambda (_)
-                                 (execute-kbd-macro
-                                  (kbd (concat "M-" (number-to-string index))))))))
-    (concat
-     (propertize (number-to-string index) 'face 'blue-hint-index)
-     " "
-     (propertize build-dir*
-                 'face face
-                 'help-echo "Click to select build directory"))))
-
-(defun blue--create-hint-overlay (build-dirs current override)
-  "Create hint overlay content from BUILD-DIRS and CURRENT build-dir.
-
-If OVERRIDE is non nil disable CONFIGS."
-  (when-let* (build-dirs
-              (indices (number-sequence 1 (length build-dirs)))
-              (formatted (seq-mapn (lambda (idx dir)
-                                     (blue--format-build-dir-hint idx dir current))
-                                   indices build-dirs))
-              (lines (string-join formatted "\n"))
-              (lines* (if override
-                          (concat
-                           (blue--format-build-dir-hint 0 override override)
-                           "\n"
-                           (propertize lines 'face 'blue-hint-separator))
-                        lines)))
-    (concat
-     "Build directory (M-<num> to select):\n"
-     lines*
-     "\n"
-     (propertize " " 'face 'blue-hint-separator
-                 'display '(space :align-to right))
-     "\n")))
-
-(defun blue--show-hints (&rest _)
-  "Display build directory hints in minibuffer overlay."
-  (when-let* ((build-dirs (blue--cache-get-build-dirs blue--blueprint))
-              (content (blue--create-hint-overlay build-dirs (blue--get-build-dir) blue--overiden-build-dir)))
-    (unless blue--hint-overlay
-      (setq blue--hint-overlay (make-overlay (point) (point))))
-    (overlay-put blue--hint-overlay 'after-string content)
-    (move-overlay blue--hint-overlay (point-min) (point-min) (current-buffer))))
-
-
-;;; Minibuffer Setup.
-
-(defun blue--bind-build-dir-key (index)
-  "Setup keybinding for build directory INDEX."
-  (let ((key (kbd (format "M-%d" index))))
-    (define-key (current-local-map) key
-                (lambda ()
-                  (interactive)
-                  (setq blue--build-dir
-                        (nth (1- index) (blue--cache-get-build-dirs blue--blueprint))
-                        default-directory blue--build-dir) ; Make completion work from selected build dir.
-                  (blue--show-hints)))))
-
-(defun blue--setup-minibuffer ()
-  "Setup keybindings and completion for minibuffer prompt."
-  ;; Work on a copy of the current minibuffer keymap so it doesn’t leak
-  (use-local-map (copy-keymap (current-local-map)))
-  (define-key (current-local-map) (kbd "SPC") nil)
-  (unless blue--overiden-build-dir
-    (let ((build-dirs (blue--cache-get-build-dirs blue--blueprint)))
-      (seq-do #'blue--bind-build-dir-key (number-sequence 1 (length build-dirs)))))
-  (add-hook 'completion-at-point-functions #'blue--completion-at-point nil t)
-  (blue--show-hints))
+(defun blue--get-option-from-label (label command)
+  "Retrieve option from COMMAND that matches LABEL."
+  (let ((options (blue--command-get-slot 'options command)))
+    (seq-find (lambda (option)
+                (let ((long-labels (blue--get-option-long-labels option)))
+                  (member label long-labels)))
+              options)))
 
 
 ;;; Compilation.
@@ -627,34 +539,53 @@ A comand is considered interactive if it is a member of
 
 ;;; Command Prompt.
 
-(defun blue--create-annotation-fn (commands width)
-  "Create annotation function for COMMANDS with WIDTH alignment."
-  (lambda (candidate)
-    (when-let* ((entry (seq-find (lambda (cmd)
-                                   (string= candidate (alist-get 'invoke cmd)))
-                                 commands))
-                (synopsis (alist-get 'synopsis entry)))
-      (concat (make-string (+ blue-annotation-padding
-                              (- width (string-width candidate)))
-                           ?\s)
-              (propertize synopsis 'face 'blue-documentation)))))
-
 (defun blue--create-group-fn (commands)
   "Create group function for COMMANDS."
   (lambda (candidate transform)
     (if transform
         candidate
-      (when-let* ((entry (seq-find (lambda (cmd)
-                                     (string= candidate (alist-get 'invoke cmd)))
-                                   commands))
-                  (category (alist-get 'category entry)))
-        category))))
+      (when-let* ((invocation (intern candidate))
+                  (command (blue--get-command invocation commands)))
+        (blue--command-get-slot 'category command)))))
 
-(defun blue--create-completion-properties (commands invocations)
+(defun blue--command-completion-properties (commands)
   "Create completion properties for COMMANDS and INVOCATIONS."
-  (let ((width (if invocations (apply #'max (mapcar #'string-width invocations)) 0)))
-    (list :annotation-function (blue--create-annotation-fn commands width)
-          :group-function (blue--create-group-fn commands))))
+  (when-let* ((invocations (blue--get-command-invocations commands))
+              (max-width (apply #'max (mapcar #'string-width invocations)))
+              (annotation-function
+               (lambda (candidate)
+                 (when-let* ((invocation (intern candidate))
+                             (command (blue--get-command invocation commands))
+                             (synopsis (blue--command-get-slot 'synopsis command))
+                             (padding (max (+ blue-annotation-padding
+                                              (- max-width (string-width candidate)))
+                                           2)))
+                   (concat (make-string padding ?\s)
+                           (propertize synopsis 'face 'blue-documentation)))))
+              (options-doc-buffer-function
+               (lambda (candidate)
+                 (when-let* ((invocation (intern candidate))
+                             (command (blue--get-command invocation commands))
+                             (help-msg (blue--command-get-slot 'help command)))
+                   (with-current-buffer (get-buffer-create "*blue-capf-doc*")
+                     (erase-buffer)
+                     (insert help-msg)
+                     (font-lock-add-keywords
+                      nil
+                      `(("\\[.+\\]\s*\\.\\{3\\}*" . 'blue-documentation)))
+                     (font-lock-mode 1)
+                     (font-lock-ensure)
+                     (current-buffer))))))
+    (list
+     :annotation-function annotation-function
+     :company-doc-buffer options-doc-buffer-function
+     :company-kind (lambda (candidate)
+                     (cond
+                      ((string-prefix-p "--" candidate) 'property)
+                      ((member candidate invocations) 'command)
+                      (t 'event)))
+     :exclusive 'no
+     :group-function (blue--create-group-fn commands))))
 
 (defun blue--prompt-dir (&optional create-p mustmatch)
   "Prompt for directory.
@@ -671,36 +602,6 @@ MUSTMATCH is passed directly to `read-directory-name'."
       (mkdir dir t))
     dir))
 
-(defun blue--prompt-for-commands ()
-  "Interactive prompt for BLUE commands."
-  (blue--check-blue-binary)
-  (blue--ensure-cache)
-  (setq blue--blueprint (blue--find-blueprint))
-  (let* ((prefix (car current-prefix-arg))
-         (build-dirs (blue--cache-get-build-dirs blue--blueprint))
-         (last-build-dir (car build-dirs))
-         (prompt-dir-p (or (eql prefix 4) ; Single universal argument 'C-u'.
-                           (and blue-require-build-directory
-                                (not last-build-dir))))
-         (comint-flip (eql prefix 16))) ; Double universal argument 'C-u C-u'.
-    (setq blue--overiden-build-dir (when prompt-dir-p
-                                     (blue--prompt-dir t))
-          blue--build-dir (or blue--overiden-build-dir last-build-dir)
-          blue--data (blue--get-data blue--blueprint))
-    ;; Make completion work from selected build dir.
-    (blue--set-default-directory blue--build-dir)
-    (if-let* ((commands (car blue--data))
-              (invocations (mapcar (lambda (cmd) (alist-get 'invoke cmd)) commands))
-              (completion-extra-properties
-               (blue--create-completion-properties commands invocations))
-              (crm-separator (propertize "[ \t]*--[ \t]+"
-                                         'separator "--"
-                                         'description "double-dash-separated list"))
-              (crm-prompt "[%d] [CMR%s] Command: "))
-        (list (minibuffer-with-setup-hook #'blue--setup-minibuffer
-                (completing-read-multiple "Command: " invocations))
-              comint-flip)
-      (list nil))))
 
 
 ;;; UI.
@@ -744,41 +645,5 @@ MUSTMATCH is passed directly to `read-directory-name'."
     (setq blue--cache-list updated-cache)
     (blue--write-cache)))
 
-;;;###autoload
-(defun blue-run-command (input &optional comint-flip)
-  "Run a BLUE command interactively.
-
-The prompt will hint for a directory where to run the BLUE command in a
-directory.  The hinted directories are directories where BLUE has been
-previously executed by `blue.el'.  The execution directory can be
-changed using `M-<num>'.
-
-Invoked with universal prefix argument '\\[universal-argument]', prompt
-for a directory to use when running `blue'.
-
-Invoked with double universal prefix argument '\\[universal-argument]
-\\[universal-argument]', invert the interactive heuristics configured by
-`blue-interactive-commands'.
-
-INPUT is a list of command strings.
-COMINT-FLIP inverts the interactive compilation logic."
-  (interactive (blue--prompt-for-commands))
-
-  (when input
-    (let* ((options (blue--normalize-options blue-default-options))
-           (tokens (mapcar #'string-split input))
-           (is-interactive (blue--any-interactive-p tokens))
-           (comint (xor is-interactive comint-flip)))
-      (let ((command-string (string-join
-                             (cons blue-binary
-                                   (append (when options options)
-                                           (list (string-join input " -- "))))
-                             " ")))
-        ;; Bring `blue--build-dir' to the from of the list so it's ordered by
-        ;; usage.
-        (when-let* ((build-dir (blue--get-build-dir)))
-          (blue--cache-add build-dir))
-        (blue--compile command-string comint)))))
-
 (provide 'blue)
 ;;; blue.el ends here.
